@@ -148,6 +148,8 @@ class Vsys(VersionedPanObject):
         "network.VlanInterface",
         "network.Vlan",
         "network.VirtualRouter",
+        "network.LogicalRouter",
+        "network.Vrf",
         "network.VirtualWire",
         "network.Layer2Subinterface",
         "network.Layer3Subinterface",
@@ -397,6 +399,38 @@ class SystemSettings(VersionedPanObject):
         self._params = tuple(params)
 
 
+class AdvancedRoutingEngine(VersionedPanObject):
+    """
+    Note: This is valid for PANS-OS 10.2+.
+
+    Args:
+        enable (bool): Enable advanced routing engine
+
+    """
+
+    NAME = None
+    ROOT = Root.DEVICE
+    CHILDTYPES = ()
+
+    def _setup(self):
+        # xpaths
+        self._xpaths.add_profile(value="/deviceconfig/setting")
+
+        # params
+        params = []
+
+        params.append(
+            VersionedParamPath(
+                "enable",
+                default=False,
+                vartype="yesno",
+                path="advance-routing",
+            )
+        )
+
+        self._params = tuple(params)
+
+
 class LogSettingsSystem(VersionedPanObject):
     """Firewall or Panorama device log settings system
 
@@ -757,7 +791,7 @@ class AuthenticationProfile(VersionedPanObject):
         )
         params.append(
             VersionedParamPath(
-                "failed_attempts", vartype="int", path="locakout/failed-attempts"
+                "failed_attempts", vartype="int", path="lockout/failed-attempts"
             )
         )
         params.append(
@@ -1290,7 +1324,11 @@ class LdapServerProfile(VersionedPanObject):
             )
         )
         params.append(
-            VersionedParamPath("disabled", vartype="yesno", path="disabled",),
+            VersionedParamPath(
+                "disabled",
+                vartype="yesno",
+                path="disabled",
+            ),
         )
 
         self._params = tuple(params)
@@ -1436,7 +1474,10 @@ class SyslogServer(VersionedPanObject):
                 "facility",
                 default="LOG_USER",
                 path="facility",
-                values=["LOG_USER",] + ["LOG_LOCAL{0}".format(x) for x in range(8)],
+                values=[
+                    "LOG_USER",
+                ]
+                + ["LOG_LOCAL{0}".format(x) for x in range(8)],
             )
         )
 
@@ -1494,6 +1535,12 @@ class HttpServerProfile(VersionedPanObject):
         iptag_name (str): (PAN-OS 9.0+) Name for custom IP tag format
         iptag_uri_format (str): (PAN-OS 9.0+) URI format for custom IP tag format
         iptag_payload (str): (PAN-OS 9.0+) Payload for custom IP tag format
+        globalprotect_name (str): (PAN-OS 9.1+) Name for custom GlobalProtect format
+        globalprotect_uri_format (str): (PAN-OS 9.1+) URI format for custom GlobalProtect format
+        globalprotect_payload (str): (PAN-OS 9.1+) Payload for custom GlobalProtect format
+        decryption_name (str): (PAN-OS 10.0+) Name for custom Decryption format
+        decryption_uri_format (str): (PAN-OS 10.0+) URI format for custom Decryption format
+        decryption_payload (str): (PAN-OS 10.0+) Payload for custom Decryption format
 
     """
 
@@ -1529,6 +1576,10 @@ class HttpServerProfile(VersionedPanObject):
         "device.HttpSctpParam",
         "device.HttpIpTagHeader",
         "device.HttpIpTagParam",
+        "device.HttpGlobalProtectHeader",
+        "device.HttpGlobalProtectParam",
+        "device.HttpDecryptionHeader",
+        "device.HttpDecryptionParam",
     )
 
     def _setup(self):
@@ -1635,6 +1686,18 @@ class HttpServerProfile(VersionedPanObject):
         params[-1].add_profile("9.0.0", path="format/iptag/url-format")
         params.append(VersionedParamPath("iptag_payload", exclude=True))
         params[-1].add_profile("9.0.0", path="format/iptag/payload")
+        params.append(VersionedParamPath("globalprotect_name", exclude=True))
+        params[-1].add_profile("9.1.0", path="format/globalprotect/name")
+        params.append(VersionedParamPath("globalprotect_uri_format", exclude=True))
+        params[-1].add_profile("9.1.0", path="format/globalprotect/url-format")
+        params.append(VersionedParamPath("globalprotect_payload", exclude=True))
+        params[-1].add_profile("9.1.0", path="format/globalprotect/payload")
+        params.append(VersionedParamPath("decryption_name", exclude=True))
+        params[-1].add_profile("10.0.0", path="format/decryption/name")
+        params.append(VersionedParamPath("decryption_uri_format", exclude=True))
+        params[-1].add_profile("10.0.0", path="format/decryption/url-format")
+        params.append(VersionedParamPath("decryption_payload", exclude=True))
+        params[-1].add_profile("10.0.0", path="format/decryption/payload")
 
         self._params = tuple(params)
 
@@ -2113,6 +2176,66 @@ class HttpIpTagParam(ValueEntry):
     ROOT = Root.PANORAMA_VSYS
 
 
+class HttpDecryptionHeader(ValueEntry):
+    """HTTP header for Decryption.
+
+    Note: This is valid for PAN-OS 10.0+
+
+    Args:
+        name (str): The header name
+        value (str): The header value
+
+    """
+
+    LOCATION = "/format/decryption/headers"
+    ROOT = Root.PANORAMA_VSYS
+
+
+class HttpDecryptionParam(ValueEntry):
+    """HTTP param for Decryption.
+
+    Note: This is valid for PAN-OS 10.0+
+
+    Args:
+        name (str): The param name
+        value (str): The param value
+
+    """
+
+    LOCATION = "/format/decryption/params"
+    ROOT = Root.PANORAMA_VSYS
+
+
+class HttpGlobalProtectHeader(ValueEntry):
+    """HTTP header for GlobalProtect.
+
+    Note: This is valid for PAN-OS 9.1+
+
+    Args:
+        name (str): The header name
+        value (str): The header value
+
+    """
+
+    LOCATION = "/format/globalprotect/headers"
+    ROOT = Root.PANORAMA_VSYS
+
+
+class HttpGlobalProtectParam(ValueEntry):
+    """HTTP param for GlobalProtect.
+
+    Note: This is valid for PAN-OS 9.1+
+
+    Args:
+        name (str): The param name
+        value (str): The param value
+
+    """
+
+    LOCATION = "/format/globalprotect/params"
+    ROOT = Root.PANORAMA_VSYS
+
+
 class CertificateProfile(VersionedPanObject):
     """Certificate profile object.
 
@@ -2162,12 +2285,28 @@ class CertificateProfile(VersionedPanObject):
         )
         params.append(
             VersionedParamPath(
-                "username_field_value", path="username-field/{username_field}",
+                "username_field_value",
+                path="username-field/{username_field}",
+            )
+        )
+        params.append(
+            VersionedParamPath(
+                "domain",
+                path="domain",
+            )
+        )
+        params.append(
+            VersionedParamPath(
+                "use_crl",
+                path="use-crl",
+            )
+        )
+        params.append(
+            VersionedParamPath(
+                "use_ocsp",
+                path="use-ocsp",
             )
         )
-        params.append(VersionedParamPath("domain", path="domain",))
-        params.append(VersionedParamPath("use_crl", path="use-crl",))
-        params.append(VersionedParamPath("use_ocsp", path="use-ocsp",))
         params.append(
             VersionedParamPath(
                 "crl_receive_timeout",
@@ -2194,27 +2333,50 @@ class CertificateProfile(VersionedPanObject):
         )
         params.append(
             VersionedParamPath(
-                "block_unknown_certificate", vartype="yesno", path="block-unknown-cert",
+                "block_unknown_certificate",
+                vartype="yesno",
+                path="block-unknown-cert",
             )
         )
         params.append(
             VersionedParamPath(
-                "block_certificate_timeout", vartype="yesno", path="block-timeout-cert",
+                "block_certificate_timeout",
+                vartype="yesno",
+                path="block-timeout-cert",
             )
         )
         params.append(
-            VersionedParamPath("block_unauthenticated_certificate", exclude=True,)
+            VersionedParamPath(
+                "block_unauthenticated_certificate",
+                exclude=True,
+            )
         )
         params[-1].add_profile(
-            "7.1.0", vartype="yesno", path="block-unauthenticated-cert",
+            "7.1.0",
+            vartype="yesno",
+            path="block-unauthenticated-cert",
         )
-        params.append(VersionedParamPath("block_expired_certificate", exclude=True,))
-        params[-1].add_profile(
-            "8.1.0", vartype="yesno", path="block-expired-cert",
+        params.append(
+            VersionedParamPath(
+                "block_expired_certificate",
+                exclude=True,
+            )
         )
-        params.append(VersionedParamPath("ocsp_exclude_nonce", exclude=True,))
         params[-1].add_profile(
-            "9.0.0", path="ocsp-exclude-nonce", vartype="yesno",
+            "8.1.0",
+            vartype="yesno",
+            path="block-expired-cert",
+        )
+        params.append(
+            VersionedParamPath(
+                "ocsp_exclude_nonce",
+                exclude=True,
+            )
+        )
+        params[-1].add_profile(
+            "9.0.0",
+            path="ocsp-exclude-nonce",
+            vartype="yesno",
         )
 
         self._params = tuple(params)
@@ -2241,13 +2403,27 @@ class CertificateProfileCaCertificate(VersionedPanObject):
         # params
         params = []
 
-        params.append(VersionedParamPath("default_ocsp_url", path="default-ocsp-url",))
         params.append(
-            VersionedParamPath("ocsp_verify_certificate", path="ocsp-verify-cert",)
+            VersionedParamPath(
+                "default_ocsp_url",
+                path="default-ocsp-url",
+            )
+        )
+        params.append(
+            VersionedParamPath(
+                "ocsp_verify_certificate",
+                path="ocsp-verify-cert",
+            )
+        )
+        params.append(
+            VersionedParamPath(
+                "template_name",
+                exclude=True,
+            )
         )
-        params.append(VersionedParamPath("template_name", exclude=True,))
         params[-1].add_profile(
-            "9.0.0", path="template-name",
+            "9.0.0",
+            path="template-name",
         )
 
         self._params = tuple(params)
@@ -2282,7 +2458,8 @@ class SslDecrypt(VersionedPanObject):
 
         params.append(
             VersionedParamPath(
-                "forward_trust_certificate_rsa", path="forward-trust-certificate/rsa",
+                "forward_trust_certificate_rsa",
+                path="forward-trust-certificate/rsa",
             )
         )
         params.append(
@@ -2305,13 +2482,17 @@ class SslDecrypt(VersionedPanObject):
         )
         params.append(
             VersionedParamPath(
-                "root_ca_excludes", vartype="member", path="root-ca-exclude-list",
+                "root_ca_excludes",
+                vartype="member",
+                path="root-ca-exclude-list",
             )
         )
         # Only option present on Panorama.
         params.append(
             VersionedParamPath(
-                "trusted_root_cas", vartype="member", path="trusted-root-CA",
+                "trusted_root_cas",
+                vartype="member",
+                path="trusted-root-CA",
             )
         )
         params.append(
@@ -2346,8 +2527,19 @@ class SslDecryptExcludeCert(VersionedPanObject):
         # params
         params = []
 
-        params.append(VersionedParamPath("description", path="description",))
-        params.append(VersionedParamPath("exclude", vartype="yesno", path="exclude",))
+        params.append(
+            VersionedParamPath(
+                "description",
+                path="description",
+            )
+        )
+        params.append(
+            VersionedParamPath(
+                "exclude",
+                vartype="yesno",
+                path="exclude",
+            )
+        )
 
         self._params = tuple(params)
 
@@ -2377,9 +2569,19 @@ class LocalUserDatabaseUser(VersionedPanObject):
         params = []
 
         params.append(
-            VersionedParamPath("password_hash", vartype="encrypted", path="phash",)
+            VersionedParamPath(
+                "password_hash",
+                vartype="encrypted",
+                path="phash",
+            )
+        )
+        params.append(
+            VersionedParamPath(
+                "disabled",
+                vartype="yesno",
+                path="disabled",
+            )
         )
-        params.append(VersionedParamPath("disabled", vartype="yesno", path="disabled",))
 
         self._params = tuple(params)
 
@@ -2420,6 +2622,12 @@ class LocalUserDatabaseGroup(VersionedPanObject):
         # params
         params = []
 
-        params.append(VersionedParamPath("users", vartype="member", path="user",))
+        params.append(
+            VersionedParamPath(
+                "users",
+                vartype="member",
+                path="user",
+            )
+        )
 
         self._params = tuple(params)
