@@ -718,7 +718,13 @@ class UserId(object):
                 te = entry.find("./tag")
                 break
         else:
-            entry = ET.SubElement(ru, "entry", {"user": user,})
+            entry = ET.SubElement(
+                ru,
+                "entry",
+                {
+                    "user": user,
+                },
+            )
             te = ET.SubElement(entry, "tag")
 
         # Now add in the tags with the specified timeout.
@@ -761,7 +767,13 @@ class UserId(object):
             if entry.attrib["user"] == user:
                 break
         else:
-            entry = ET.SubElement(uu, "entry", {"user": user,})
+            entry = ET.SubElement(
+                uu,
+                "entry",
+                {
+                    "user": user,
+                },
+            )
 
         # Do tag removal.
         te = entry.find("./tag")
